from __future__ import print_function

try:
    from qtpy.uic import loadUi
    from qtpy.QtWidgets import QApplication, QWidget
    gui = "qt"
except ImportError:
    gui = "none"

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg

from amuse.lab import units, SeBa, write_set_to_file
from make_a_star_cluster import make_a_star_cluster


class MakeAClusterWidget(QWidget):

    def __init__(self):

        super(MakeAClusterWidget, self).__init__()

        self.length_unit = units.parsec

        # Load file made using Qt Designer
        self.widget = loadUi('masc.ui')
        plot_area = self.widget.plot_area

        self.fig = Figure()
        self.ax = self.fig.add_subplot(1, 1, 1)
        self.setup_axes()

        self.canvas = FigureCanvasQTAgg(self.fig)
        plot_area.addWidget(self.canvas)

    def setup_axes(self):
        self.ax.set_xlim(-10, 10)
        self.ax.set_ylim(-10, 10)
        self.ax.set_xlabel(self.length_unit)
        self.ax.set_ylabel(self.length_unit)
        self.ax.set_aspect(1)

    def create_cluster(self):
        number_of_stars = self.widget.number_of_stars_box.value()
        star_metallicity = self.widget.star_metallicity_box.value()
        mass_distribution = self.widget.mass_distribution_box.itemText(
                self.widget.mass_distribution_box.currentIndex(),
                )
        star_distribution = self.widget.star_distribution_box.itemText(
                self.widget.star_distribution_box.currentIndex(),
                )

        self.particles = make_a_star_cluster(
                initial_mass_function=mass_distribution,
                number_of_stars=number_of_stars,
                star_metallicity=star_metallicity,
                star_distribution=star_distribution,
                )

        self.evolve_cluster()
        self.widget.actionSave.setEnabled(True)

    def evolve_cluster(self):
        age = self.widget.age_box.value() | units.Myr
        SE = SeBa()
        SE.particles.add_particles(self.particles)
        SE.evolve_model(age)
        self.particles.radius = SE.particles.radius
        self.particles.luminosity = SE.particles.luminosity
        SE.stop()
        self.plot_cluster()

    def plot_cluster(self):
        x = self.particles.x.value_in(self.length_unit)
        y = self.particles.y.value_in(self.length_unit)

        s = 0.5 * self.particles.luminosity.value_in(units.LSun)**0.5

        c = "black"

        self.ax.cla()
        self.setup_axes()

        self.ax.scatter(x, y, s=s, facecolor=c, lw=0, alpha=0.5)
        self.canvas.draw()

    def save_file(self):
        clusterfile = "cluster.hdf5"
        write_set_to_file(self.particles, clusterfile, "amuse")
        print("Saved to %s" % clusterfile)


# Initialize application
app = QApplication([])


widget = MakeAClusterWidget()

# Connect events to buttons
widget.widget.createButton.clicked.connect(widget.create_cluster)
widget.widget.evolveButton.clicked.connect(widget.evolve_cluster)
widget.widget.actionSave.triggered.connect(widget.save_file)

widget.widget.show()

# Start 'event loop'
app.exec_()

widget = loadUi('masc.ui')



from __future__ import print_function

# from math import cos, sin, pi
# from amuse.ext.orbital_elements import (
#         new_binary_from_orbital_elements,
#         # orbital_elements_from_binary,
#         )

from amuse.lab import (
        units, constants,
        Particles,
        # new_salpeter_mass_distribution,
        write_set_to_file,
        )
from amuse.ext.orbital_elements import (
        generate_binaries,
        )

import numpy as np
from numpy import pi
from numpy.random import (
        random,
        uniform,
        )


def circular_velocity(
        primary,
        secondary,
        ):
        v_circ = (
            constants.G * (primary.mass + secondary.mass)
            / (secondary.position-primary.position).length()
            )**0.5
        return v_circ


def orbital_period_to_semi_major_axis(
        period, mass1, mass2=0 | units.MSun, G=constants.G):
    """
    returns semi-major axis for given period and masses
    """
    mu = G * (mass1 + mass2)
    a3 = (period/(2*np.pi))**2 * mu
    return a3**(1./3)


def new_binary_distribution(
        primary_mass,
        secondary_mass=None,
        binaries=None,
        min_mass=0.08 | units.MSun,
        ):
    """
    Takes primary masses, and returns a set of stars and a set of binaries
    formed by these stars.
    Secondary masses are given by a uniform random mass ratio with the primary
    masses. If optional secondary masses are given, these are used instead.
    binaries is an optional particleset used for the binary pairs, with given
    positions and velocities. Other parameters are ignored.
    """
    N = len(primary_mass)
    if binaries is None:
        binaries = Particles(N)
        # Should give some position/velocity as well?
    elif len(binaries) != N:
        print("binaries must be None or have the same lenght as primary_mass")
        return -1
    if secondary_mass is None:
        # Now, we need to specify the mass ratio in the binaries.
        # A flat distribution seems to be OK.
        mass_ratio = uniform(N)
        # This gives us the secondaries' masses
        secondary_mass = mass_ratio * primary_mass
        # secondaries are min_mass at least
        secondary_mass = np.maximum(secondary_mass, min_mass)
    elif len(secondary_mass) != N:
        print("Number of secondaries is unequal to number of primaries!")
        return -1
    else:
        # Make sure primary_mass is the larger of the two, and secondary_mass
        # the smaller.
        pm = primary_mass.maximum(secondary_mass)
        sm = primary_mass.minimum(secondary_mass)
        primary_mass = pm
        secondary_mass = sm
        del(pm, sm)
    # Now, we need to calculate the semi-major axes for the binaries. Since the
    # observed quantity is orbital periods, we start from there.
    mean_log_orbital_period = 5  # 10log of the period in days, (Duchene&Kraus)
    sigma_log_orbital_period = 2.3
    orbital_period = np.random.lognormal(
            size=N,
            mean=np.log(10) * mean_log_orbital_period,
            sigma=np.log(10) * sigma_log_orbital_period,
            ) | units.day
    # We need the masses to calculate the corresponding semi-major axes.
    semi_major_axis = orbital_period_to_semi_major_axis(
            orbital_period,
            primary_mass,
            secondary_mass,
            )
    # Eccentricity: square root of random value
    eccentricity = np.sqrt(random(N))
    # Other orbital elements at random
    inclination = pi * random(N) | units.rad
    true_anomaly = 2 * pi * random(N) | units.rad
    longitude_of_the_ascending_node = 2 * pi * random(N) | units.rad
    argument_of_periapsis = 2 * pi * random(N) | units.rad
    primaries, secondaries = generate_binaries(
            primary_mass,
            secondary_mass,
            semi_major_axis,
            eccentricity=eccentricity,
            inclination=inclination,
            true_anomaly=true_anomaly,
            longitude_of_the_ascending_node=longitude_of_the_ascending_node,
            argument_of_periapsis=argument_of_periapsis,
            G=constants.G,
            )
    stars = Particles()

    primaries.position += binaries.position
    secondaries.position += binaries.position
    primaries.velocity += binaries.velocity
    secondaries.velocity += binaries.velocity
    primaries = stars.add_particles(primaries)
    secondaries = stars.add_particles(secondaries)

    binaries.eccentricity = eccentricity
    binaries.semi_major_axis = semi_major_axis
    for i in range(len(primaries)):
        binaries[i].child1 = primaries[i]
        binaries[i].child2 = secondaries[i]
        # Probably needed
        binaries[i].mass = primaries[i].mass + secondaries[i].mass
    return stars, binaries


if __name__ == "__main__":
    N = 4
    stars, binaries = new_binary_distribution(N, model="default")
    write_set_to_file(stars, "stars.hdf5", "amuse")
    write_set_to_file(binaries, "stars-binarypairs.hdf5", "amuse")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='amuse_masc',
    version='0.2.2',
    description='Make a star cluster',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/rieder/masc',
    author='Steven Rieder',
    author_email='steven@rieder.nl',
    license='MIT',
    packages=setuptools.find_packages(),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4',
    classifiers=[
        # Python versions supported by amuse_masc
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",

        # License
        "License :: OSI Approved :: MIT License",

        # OS support
        "Operating System :: OS Independent",

        # Maturity of amuse_masc
        "Development Status :: 4 - Beta",

        # Intended audience
        "Intended Audience :: Science/Research",
    ],
)



#!/usr/bin/env python
"""
make_a_star_cluster.py creates a model star cluster,
which can then be used in N-body simulations or for other purposes.

It requires AMUSE, which can be downloaded from http://amusecode.org or
https://github.com/amusecode/amuse.

Currently not feature-complete yet, and function/argument names are
subject to change.

-- Steven Rieder steven at rieder punt nl
"""
from __future__ import print_function, division

import os
import argparse

import numpy as np

# from amuse.lab import *
from amuse.io import write_set_to_file
from amuse.lab import (
    units,
    nbody_system,
    generic_unit_converter,
)
from amuse.support.console import set_printing_strategy


def new_argument_parser():
    "Parse command line arguments"
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-o',
        dest='clustername',
        default="auto",
        help="Outputfile name [auto]",
    )
    parser.add_argument(
        '-type',
        dest='filetype',
        default="amuse",
        help="Output file type ([amuse]/txt/starlab/nemo)",
        )
    parser.add_argument(
        '-N',
        dest='number_of_stars',
        default=1024,
        type=int,
        help='Number of stars [1024]',
    )
    parser.add_argument(
        '-M',
        dest='cluster_mass',
        type=float,
        default=0,
        help='Cluster mass (takes precedence over number of stars)',
    )
    parser.add_argument(
        '-dist',
        dest='star_distribution',
        default="plummer",
        help="Star distribution ([plummer]/king/fractal)",
    )
    parser.add_argument(
        '-gasdist',
        dest='gas_distribution',
        default="none",
        help="Gas distribution ([none]/plummer/king/fractal) \
                NOT IMPLEMENTED YET",
    )
    parser.add_argument(
        '-imf',
        dest='initial_mass_function',
        default="kroupa",
        help="IMF ([kroupa]/salpeter/fixed)",
    )
    parser.add_argument(
        '-modelnr',
        dest='cluster_model_number',
        type=int,
        default=30483,
        help='Model number',
    )
    parser.add_argument(
        '-kingw',
        dest='king_parameter_w0',
        type=float,
        default=7.0,
        help='King W0 parameter [7.0]',
    )
    parser.add_argument(
        '-fractald',
        dest='fractal_parameter_fd',
        type=float,
        default=2.0,
        help='Fractal distribution [2.0]',
    )
    parser.add_argument(
        '-uml',
        dest='upper_mass_limit',
        type=float,
        default=125.,
        help='Upper mass limit (in MSun) [125.]',
    )
    parser.add_argument(
        '-Z',
        dest='metallicity',
        type=float,
        default=0.01,
        help='Metallicity [0.01]',
    )
    parser.add_argument(
        '-ibf',
        dest='initial_binary_fraction',
        type=float,
        default=0.,
        help='Initial binary fraction (0-1) [0.] NOT IMPLEMENTED YET',
    )
    parser.add_argument(
        '-R',
        dest='effective_radius',
        type=float,
        default=3.0,
        help='Cluster radius [3.0] (parsec)',
    )
    parser.add_argument(
        '-vr',
        dest='virial_ratio',
        type=float,
        default=0.5,
        help="Virial ration [0.5], 0.5=stable, 0.75=just expelled gas, \
                0.1=collapsing",
    )
    args = parser.parse_args()
    return args


def new_cluster(
        stellar_mass=False,
        initial_mass_function="salpeter",
        upper_mass_limit=125. | units.MSun,
        number_of_stars=1024,
        effective_radius=3.0 | units.parsec,
        star_distribution="plummer",
        star_distribution_w0=7.0,
        star_distribution_fd=2.0,
        star_metallicity=0.01,
        # initial_binary_fraction=0,
        **kwargs
):
    """
    Create stars.
    When using an IMF, either the stellar mass is fixed (within
    stochastic error) or the number of stars is fixed. When using
    equal-mass stars, both are fixed.
    """

    if stellar_mass:
        # Add stars to cluster, until mass limit reached (inclusive!)
        if initial_mass_function == "kroupa":
            from amuse.ic.brokenimf import new_kroupa_mass_distribution
            mass = new_kroupa_mass_distribution(0)
            while mass.sum() < stellar_mass:
                mass.append(
                    new_kroupa_mass_distribution(
                        1,
                        mass_max=upper_mass_limit,
                        )[0]
                )
            total_mass = mass.sum()
            number_of_stars = len(mass)
        elif initial_mass_function == "salpeter":
            from amuse.ic.salpeter import new_salpeter_mass_distribution
            mass = new_salpeter_mass_distribution(0)
            while mass.sum() < stellar_mass:
                mass.append(
                    new_salpeter_mass_distribution(
                        1,
                        mass_max=upper_mass_limit,
                        )[0]
                )
            total_mass = mass.sum()
            number_of_stars = len(mass)
        elif initial_mass_function == "fixed":
            mass_of_individual_star = stellar_mass / number_of_stars
            mass = mass_of_individual_star
            total_mass = stellar_mass
        else:
            return -1, "No mass function"
    else:
        # Give stars their mass
        if initial_mass_function == "kroupa":
            from amuse.ic.brokenimf import new_kroupa_mass_distribution
            mass = new_kroupa_mass_distribution(
                number_of_stars,
                mass_max=upper_mass_limit,
            )
            total_mass = mass.sum()
        elif initial_mass_function == "salpeter":
            from amuse.ic.salpeter import new_salpeter_mass_distribution
            mass = new_salpeter_mass_distribution(
                number_of_stars,
                mass_max=upper_mass_limit,
            )
            total_mass = mass.sum()
        elif initial_mass_function == "fixed":
            mass = mass_of_individual_star
            total_mass = number_of_stars * mass
        else:
            return -1, "No mass function"

    converter = generic_unit_converter.ConvertBetweenGenericAndSiUnits(
        total_mass,
        1. | units.kms,
        effective_radius,
    )
    # Give stars a position and velocity, based on the distribution model.
    if star_distribution == "plummer":
        from amuse.ic.plummer import new_plummer_sphere
        stars = new_plummer_sphere(
            number_of_stars,
            convert_nbody=converter,
        )
    elif star_distribution == "king":
        from amuse.ic.kingmodel import new_king_model
        stars = new_king_model(
            number_of_stars,
            star_distribution_w0,
            convert_nbody=converter,
        )
    elif star_distribution == "fractal":
        from amuse.ic.fractalcluster import new_fractal_cluster_model
        stars = new_fractal_cluster_model(
            number_of_stars,
            fractal_dimension=star_distribution_fd,
            convert_nbody=converter,
        )
    else:
        return -1, "No stellar distribution"

    # set the stellar mass.
    stars.mass = mass

    # set other stellar parameters.
    stars.metallicity = star_metallicity

    # Virialize the star cluster
    stars.move_to_center()
    stars.scale_to_standard(
        convert_nbody=converter,
        # virial_ratio=virial_ratio,
        # smoothing_length_squared= ...,
    )

    # Record the cluster's initial parameters to the particle distribution
    stars.collection_attributes.initial_mass_function = initial_mass_function
    stars.collection_attributes.upper_mass_limit = upper_mass_limit
    stars.collection_attributes.number_of_stars = number_of_stars

    stars.collection_attributes.effective_radius = effective_radius

    stars.collection_attributes.star_distribution = star_distribution
    stars.collection_attributes.star_distribution_w0 = star_distribution_w0
    stars.collection_attributes.star_distribution_fd = star_distribution_fd

    stars.collection_attributes.star_metallicity = star_metallicity

    # Derived/legacy values
    stars.collection_attributes.converter_mass = \
        converter.to_si(1 | nbody_system.mass)
    stars.collection_attributes.converter_length =\
        converter.to_si(1 | nbody_system.length)
    stars.collection_attributes.converter_speed =\
        converter.to_si(1 | nbody_system.speed)

    return stars


def main():
    "Make a star cluster"
    set_printing_strategy(
        "custom",
        preferred_units=[units.MSun, units.parsec, units.yr, units.kms],
        precision=5,
    )
    clustertemplate = "TESTCluster_%08i"

    args = new_argument_parser()
    cluster_model_number = args.cluster_model_number
    star_distribution = args.star_distribution
    # gas_distribution = args.gas_distribution
    king_parameter_w0 = args.king_parameter_w0
    fractal_parameter_fd = args.fractal_parameter_fd
    initial_mass_function = args.initial_mass_function
    number_of_stars = args.number_of_stars
    if args.cluster_mass != 0:
        cluster_mass = args.cluster_mass | units.MSun
    else:
        cluster_mass = False
    upper_mass_limit = args.upper_mass_limit | units.MSun
    effective_radius = args.effective_radius | units.parsec
    metallicity = args.metallicity
    # virial_ratio = args.virial_ratio
    filetype = args.filetype

    # not implemented yet
    # initial_binary_fraction = args.initial_binary_fraction

    np.random.seed(cluster_model_number)

    if not (number_of_stars or cluster_mass):
        print("no number of stars or cluster mass given, exiting")
        exit()

    stars = new_cluster(
        stellar_mass=cluster_mass,
        initial_mass_function=initial_mass_function,
        upper_mass_limit=upper_mass_limit,
        number_of_stars=number_of_stars,
        effective_radius=effective_radius,
        star_distribution=star_distribution,
        star_distribution_w0=king_parameter_w0,
        star_distribution_fd=fractal_parameter_fd,
        star_metallicity=metallicity,
    )

    print(stars.mass.sum())
    print(len(stars))

    if args.clustername != "auto":
        clustername = args.clustername
    else:
        cluster_file_exists = True
        N = -1
        while cluster_file_exists:
            N += 1
            clustername = (
                clustertemplate % N
                + "." + filetype
            )
            cluster_file_exists = os.path.isfile(clustername)

    write_set_to_file(stars, clustername, filetype)


if __name__ in ["__main__"]:
    main()
